//******************************************************************************
// Programa MEMO_CTRL_24C01.C - exemplo de uso para banco de memria com 24C01
//
// Permite o controle de at 8 CIs: com 24C01 banco = 1 kB
//                                      24C02 banco = 2 kB
//
// Para usar 24C02 necessrio alterar valor de TAM_EEPROM em memo_ctrl_24C01.h
// para 2048.
//
// Obs.: Programa no admite uso de 24C01 e 24C02 no mesmo banco.
//
// Desenvolvido por Mrcio Jos Soares
// para Editora Saber
//
// Usa a INT do TIMER1 para piscar LED na frequencia de 1Hz
// Usa a INT da USART para receber um byte
// Verso trabalha com Watch Dog Timer ligado
// Clock interno ativado em 4MHz
//
// Microcontrolador utilizado: PIC16F628-04/P
// Compilador: CCS C Compiler - verso PCM 3.15 - IDE verso 3.28
//
// Pinos utilizados:
// RA0 - SCK banco de memria
// RA1 - SDA banco de memria
// RA2 - no utilizado
// RA3 - no utilizado
// RA4 - LED
// RA5 - no utilizado
// RA6 - no utilizado
// RA7 - no utilizado
// RB0 - no utilizado
// RB1 - RX para RS-232
// RB2 - TX para RS-232
// RB3 - no utilizado
// RB4 - no utilizado
// RB5 - no utilizado
// RB6 - no utilizado
// RB7 - no utilizado
//
// Obs: utilizado MAX232 para comunicao com PC
//
//******************************************************************************

//******************************************************************************
// Este cdigo-fonte  parte integrante do artigo Banco de memria I2C com
// microcontrolador PIC. Maiores informaes sobre o projeto podem ser obtidas
// no referido artigo publicado na Revista Eletrnica Total
//******************************************************************************

//******************************************************************************
// O Copyright deste programa est reservado para Mrcio Jos Soares e seu
// contedo est protegido pela Lei de Direitos Autorais LEI N 9.610,
// de 19 de Fevereiro de 1998.  estritamente proibida o uso comercial como
// tambm a publicao total ou parcial dos contedos aqui apresentados
// sem prvia autorizao, por escrito, do autor.
//******************************************************************************

//******************************************************************************
// Arquivos incluidos
//******************************************************************************
#include "memo_ctrl_24C01.h"     //diretrizes de pr-compilao do mdulo
#include "2401.c"                //biblioteca de uso para memo 2401

//******************************************************************************
// Funes do mdulo
//******************************************************************************

//******************************************************************************
// Funo env_msg_inic - Envia mensagem de inicializao e menu
//
// Entradas - nenhuma
// Saidas   - nenhuma
//******************************************************************************
void env_msg_inic(){

   int i;

   for (i=0;i<sizeof(msg1)-1;i++){
      printf("%c",msg1[i]);
    	restart_WDT();						//zera WDT
   }

   for (i=0;i<sizeof(msg2)-1;i++){
      printf("%c",msg2[i]);
    	restart_WDT();						//zera WDT
   }
   printf("%c\)", banco + 0x30);

   for (i=0;i<sizeof(msg3)-1;i++){
      printf("%c",msg3[i]);
    	restart_WDT();						//zera WDT
   }

   for (i=0;i<sizeof(msg4)-1;i++){
      printf("%c",msg4[i]);
    	restart_WDT();						//zera WDT
   }

   for (i=0;i<sizeof(msg5)-1;i++){
      printf("%c",msg5[i]);
    	restart_WDT();						//zera WDT
   }

   for (i=0;i<sizeof(msg6)-1;i++){
      printf("%c",msg6[i]);
    	restart_WDT();						//zera WDT
   }

   for (i=0;i<sizeof(msg7)-1;i++){
      printf("%c",msg7[i]);
    	restart_WDT();						//zera WDT
   }

   for (i=0;i<sizeof(msg8)-1;i++){
      printf("%c",msg8[i]);
    	restart_WDT();						//zera WDT
   }

}

//******************************************************************************
// Funo para tratar a interrupo para dado recebido RS-232
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_RDA
RDA_isr(){

   static char carac;
   byte i,j;

   restart_WDT();                //restaura WDT

   carac = rxreg;                //colhe dado do canal serial
   printf("%c", carac);          //envia eco do caracter digitado

   switch (maq_est){             //maquina de estado que determina operao
      case 0:                    //banco selecionado (default 0)
         switch (carac){
            case '1':
               printf ("\n\r\n\rSelecione o banco (0 a %1u)->",NCIS-1);
               restart_WDT();
               maq_est = 1;
               break;
            case '2':            //apaga banco
               printf ("\n\r\n\rApagando banco->%c", banco + 0x30);
               restart_WDT();

               for (i=0; i<128; i++)
                  write_ext_eeprom(0xA0 | (banco<<1), i, 0xFF);

               printf ("\n\rBanco apagado");
               restart_WDT();
               break;
            case '3':            //grava dado
               printf ("\n\r\n\rEntre com o endereo (use ALT01 a ALT%3u) ->",(int)(TAM_EEPROM/8));
               restart_WDT();
               maq_est = 2;
               break;
            case '4':            //apaga dado
               printf ("\n\r\n\rEntre com o endereo (use ALT01 a ALT%3u) ->",(int)(TAM_EEPROM/8));
               restart_WDT();
               maq_est = 4;
               break;
            case '5':            //dump da memria em ASCII
               printf("\n\rDump do banco->%c (ASCII)", banco + 0x30);
               restart_WDT();
               for (i=0;i<LINHAS;i++){
                  printf("\n\r");
                  restart_WDT();
                  for (j=i*COLUNAS; j<COLUNAS*(i+1); j++){
                     printf("%2c  ", read_ext_eeprom(0xA1 | (banco<<1), j));
                     delay_ms(5);
                     restart_WDT();
                  }
               }
               break;
            case '6':            //dump da memria em HEXA
               printf("\n\rDump do banco->%c (HEXA)", banco + 0x30);
               restart_WDT();
               for (i=0;i<LINHAS;i++){
                  printf("\n\r");
                  restart_WDT();
                  for (j=i*COLUNAS; j<COLUNAS*(i+1); j++){
                     printf("%2X  ", read_ext_eeprom(0xA1 | (banco<<1), j));
                     delay_ms(5);
                     restart_WDT();
                  }
               }
               break;
         }
      break;

      case 1:                    //aguarda caracter para seleo do banco
         if ((carac >= 0x30) && (carac <= 0x30+(NCIS-1))){
            banco = (carac - 0x30) & 0x0F;

            printf ("\n\rBanco selecionado->%c", banco + 0x30);
            restart_WDT();
         }
         else{
            printf ("\n\rBanco invalido!");
            restart_WDT();
            }
         maq_est = 0;
         break;

      case 2:                    //aguarda endereo para gravar
         if ((carac >= 0x01) && (carac <=(int)(TAM_EEPROM/8))){
            endereco = carac - 1;
            printf("\n\rEntre com o dado->");
            restart_WDT();
            maq_est = 3;
         }
         else{
            printf ("\n\rEndereo invalido!");
            restart_WDT();
            maq_est = 0;
         }
         break;

      case 3:                    //colhe dado e grava
         dado = carac;
         write_ext_eeprom(0xA0 | (banco<<1), endereco, dado);
         printf ("\n\rDado gravado!");
         restart_WDT();
         maq_est = 0;
         break;

      case 4:                    //aguarda endereo para apagar
         if ((carac >= 0x01) && (carac <=(int)(TAM_EEPROM/8))){
            endereco = carac - 1;
            write_ext_eeprom(0xA0 | (banco<<1), endereco, 0xFF);
         }
         else{
            printf ("\n\rEndereo invalido!");
            restart_WDT();
         }
         maq_est = 0;
         break;
   }

   if (maq_est == 0){      //se mquina de estados voltou a zero, mostra menu
      printf("\n\r\n\r");
      restart_WDT();
      env_msg_inic();
   }

   restart_WDT();                //restaura WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer1
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer1
void trata_int_t1(){

	static int conta;
	static boolean led1;

	restart_WDT();					//zera WDT

	//reinicia timer1 com 3096, mais contagem que passou
	set_timer1(3096+get_timer1());
	conta++;
	if (conta == 2){       //aguarda duas contagens -> 2 x 0,5Hz = 1Hz
		conta=0;
		led1=!led1;
		output_bit(LED,led1);
		}

	restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo principal
//
// Entadas - nenhuma
// Saidas  - nenhuma
//******************************************************************************
void main(void){

   set_tris_A(0b00000000);		    //configura pinos do port A como sada,
   set_tris_B(0b00000010);		    //configura pinos do port B como sada,
                                  //exceto RB1

   setup_comparator(nc_nc_nc_nc); //desabilita comparadores internos
	setup_wdt(WDT_2304MS);			 //habilita watch dog timer a cada 2 segundos
	restart_WDT();						//zera WDT

   //timer 1 habilitado
	// [(4MHz/4)/8]/65636 = 1.90Hz -> menor frequencia possvel
	//valor inicial do timer1 (65536-62500)=3036
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
	set_timer1(3036);
	restart_WDT();								//zera WDT

	//timer2 desabilitado
	setup_timer_2(T2_DISABLED,0,1);
	restart_WDT();								//zera WDT

   //habilita interrupes
   enable_interrupts(INT_RDA);         //habilita interrupo RS-232 (rec)
   enable_interrupts(INT_TIMER1);      //habilita interrupo do timer1
   enable_interrupts(GLOBAL);          //habilita interrupo global
   restart_WDT();							   //zera WDT

   maq_est = 0;                        //inicia mquina de estados
   banco = 0;
   endereco = 0;

   init_ext_eeprom();                  //inicializa banco
   delay_ms(1000);                     //aguarda 1 segundo para comear
   env_msg_inic();                     //envia mensagem de inicializao
   restart_WDT();							   //zera WDT


//******************************************************************************
// Inicio do lao principal
// O piscar do LED  controlado via interrupo do timer 1
// e a recepo do caracter da RS-232  feita via int RDA
//******************************************************************************

   while(true)                         //faz eternamente
      restart_WDT();                   //zera WDT
}

//******************************************************************************
// Fim do programa
//******************************************************************************
